/**
 * twtash.c - Twt Hash Extension implementation
 *
 * Copyright (c) 2023 Charles E. Lehner
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 **/

/*
 * Twt Hash Extension reference: https://dev.twtxt.net/doc/twthashextension.html
 * base32 implementation: https://github.com/mjg59/tpmtotp/blob/master/base32.c
 * blake2b implementation: https://packages.trisquel.org/aramo/libb2-dev
 */

#include <blake2.h>
#include <err.h>
#include <stdio.h>
#include <string.h>
#include "base32.h"

int main(int argc, char *argv[]) {
	if (argc != 4) errx(1, "usage: twtash <author> <timestamp> <text>");
	const char *author = argv[1];
	const char *timestamp = argv[2];
	const char *text = argv[3];
	blake2b_state b2b;

	int rc = blake2b_init(&b2b, 32);
	if (rc < 0) err(1, "blake2b_init");

	rc = blake2b_update(&b2b, author, strlen(author));
	rc |= blake2b_update(&b2b, "\n", 1);
	if (rc < 0) err(1, "blake2b_update");
	rc = blake2b_update(&b2b, timestamp, strlen(timestamp));
	rc |= blake2b_update(&b2b, "\n", 1);
	if (rc < 0) err(1, "blake2b_update");
	rc = blake2b_update(&b2b, text, strlen(text));
	if (rc < 0) err(1, "blake2b_update");

	uint8_t hash[32];
	rc = blake2b_final(&b2b, hash, sizeof hash);
	if (rc < 0) err(1, "blake2b_final");

	int hash_str_len = BASE32_LEN(sizeof hash);
	char hash_str[hash_str_len+1];
	base32_encode(hash, sizeof hash, hash_str);
	while (hash_str[hash_str_len-1] == '=') hash_str_len--;
	hash_str[hash_str_len] = '\0';
	char *suffix = &hash_str[hash_str_len-7];
	for (int i = 0; i < 7; i++) {
		const char c = suffix[i];
		if (c >= 'A' && c <= 'Z') suffix[i] = c + 32;
	}
	rc = puts(suffix);
	if (rc < 0) err(1, "puts");
}
